package com.hw.mes.service.impl;

import java.util.Arrays;
import java.util.List;

import com.hw.common.core.constant.MesConstants;
import com.hw.common.core.exception.ServiceException;
import com.hw.common.core.utils.DateUtils;
import com.hw.common.core.utils.uuid.Seq;
import com.hw.common.security.utils.SecurityUtils;
import com.hw.mes.domain.MesBaseAttachInfo;
import com.hw.mes.domain.MesProductOrder;
import com.hw.mes.mapper.MesBaseAttachInfoMapper;
import com.hw.mes.mapper.MesProductPlanDetailMapper;
import com.hw.mes.service.IMesBaseAttachInfoService;
import com.hw.mes.service.IMesProductOrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;

import com.hw.common.core.utils.StringUtils;
import org.springframework.transaction.annotation.Transactional;
import com.hw.mes.domain.MesProductPlanDetail;
import com.hw.mes.mapper.MesProductPlanMapper;
import com.hw.mes.domain.MesProductPlan;
import com.hw.mes.service.IMesProductPlanService;

/**
 * 生产派工Service业务层处理
 *
 * @author xins
 * @date 2024-02-21
 */
@Service
public class MesProductPlanServiceImpl implements IMesProductPlanService {
    @Autowired
    private MesProductPlanMapper mesProductPlanMapper;

    @Autowired
    private IMesProductOrderService mesProductOrderService;

    @Autowired
    private IMesBaseAttachInfoService mesBaseAttachInfoService;

    @Autowired
    private MesBaseAttachInfoMapper mesBaseAttachInfoMapper;

    @Autowired
    private MesProductPlanDetailMapper productPlanDetailMapper;

    /**
     * 查询生产派工
     *
     * @param planId 生产派工主键
     * @return 生产派工
     */
    @Override
    public MesProductPlan selectMesProductPlanByPlanId(Long planId) {
        return mesProductPlanMapper.selectMesProductPlanByPlanId(planId);
    }

    /**
     * 查询生产派工列表
     *
     * @param mesProductPlan 生产派工
     * @return 生产派工
     */
    @Override
    public List<MesProductPlan> selectMesProductPlanList(MesProductPlan mesProductPlan) {
        return mesProductPlanMapper.selectMesProductPlanList(mesProductPlan);
    }

    /**
     * 新增生产派工
     *
     * @param mesProductPlan 生产派工
     * @return 结果
     */
    @Transactional
    @Override
    public int insertMesProductPlan(MesProductPlan mesProductPlan) {
        mesProductPlan.setCreateBy(SecurityUtils.getUsername());
        mesProductPlan.setCreateTime(DateUtils.getNowDate());
        mesProductPlan.setPlanCode(Seq.getId(Seq.planCodeSeqType, Seq.planCodeCode));
        int rows = mesProductPlanMapper.insertMesProductPlan(mesProductPlan);
        //更新工单已派工数量
        MesProductOrder mesProductOrder = new MesProductOrder();
        mesProductOrder.setProductOrderId(mesProductPlan.getProductOrderId());
        mesProductOrder.setDispatchAmount(mesProductPlan.getDispatchedAmount());
        mesProductOrderService.updateMesProductOrder(mesProductOrder);
        insertMesProductPlanDetail(mesProductPlan);
        return rows;
    }

    /**
     * 修改生产派工
     *
     * @param mesProductPlan 生产派工
     * @return 结果
     */
    @Transactional
    @Override
    public int updateMesProductPlan(MesProductPlan mesProductPlan) {
        mesProductPlan.setUpdateBy(SecurityUtils.getUsername());
        mesProductPlan.setUpdateTime(DateUtils.getNowDate());
        mesProductPlanMapper.deleteMesProductPlanDetailByPlanId(mesProductPlan.getPlanId());
        insertMesProductPlanDetail(mesProductPlan);
        return mesProductPlanMapper.updateMesProductPlan(mesProductPlan);
    }

    /**
     * 批量删除生产派工
     *
     * @param planIds 需要删除的生产派工主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteMesProductPlanByPlanIds(Long[] planIds) {
        mesProductPlanMapper.deleteMesProductPlanDetailByPlanIds(planIds);
        return mesProductPlanMapper.deleteMesProductPlanByPlanIds(planIds);
    }

    /**
     * 删除生产派工信息
     *
     * @param planId 生产派工主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deleteMesProductPlanByPlanId(Long planId) {
        mesProductPlanMapper.deleteMesProductPlanDetailByPlanId(planId);
        return mesProductPlanMapper.deleteMesProductPlanByPlanId(planId);
    }

    /**
     * 新增生产计划明细信息
     *
     * @param mesProductPlan 生产派工对象
     */
    public void insertMesProductPlanDetail(MesProductPlan mesProductPlan) {
        List<MesProductPlanDetail> mesProductPlanDetailList = mesProductPlan.getMesProductPlanDetailList();
        Long planId = mesProductPlan.getPlanId();
        if (StringUtils.isNotNull(mesProductPlanDetailList)) {
            List<MesProductPlanDetail> list = new ArrayList<>();
            for (MesProductPlanDetail mesProductPlanDetail : mesProductPlanDetailList) {
                mesProductPlanDetail.setPlanId(planId);
                list.add(mesProductPlanDetail);
            }
            if (list.size() > 0) {
                mesProductPlanMapper.batchMesProductPlanDetail(list);
            }
        }
    }

    /**
     * 获取派工编号
     *
     * @return
     */
    @Override
    public String getDispatchCode() {
        return Seq.getId(Seq.dispatchCodeSeqType, Seq.dispatchCodeCode);
    }

    /**
     * 生产工单新增生产派工List
     *
     * @param mesProductPlanList 生产派工
     * @return
     */
    @Override
    public int orderAddMesProductPlanList(List<MesProductPlan> mesProductPlanList) {
        for (MesProductPlan mesProductPlan : mesProductPlanList) {
            this.insertMesProductPlan(mesProductPlan);
        }
        return 1;
    }

    /**
     * 获取生产派工附件List列表
     * @param planId
     * @param attachType
     * @return
     */
    @Override
    public List<MesBaseAttachInfo> getDispatchAttachList(Long planId, String attachType) {
        MesProductPlan mesProductPlan = mesProductPlanMapper.selectMesProductPlanByPlanId(planId);
        try {
            if (attachType.equals(MesConstants.MES_ATTACH_TYPE_DRAWING)){
                if (StringUtils.isEmpty(mesProductPlan.getAttachId())) {
                    return new ArrayList<>();
                }
                Long[] attachIds = Arrays.stream(mesProductPlan.getAttachId().split(","))
                        .map(String::trim)
                        .map(Long::parseLong)
                        .toArray(Long[]::new);
                return mesBaseAttachInfoService.selectMesBaseAttachInfoByAttachIds(attachIds);
            } else if (attachType.equals(MesConstants.MES_ATTACH_TYPE_SOP)) {
                if (StringUtils.isEmpty(mesProductPlan.getSopId())) {
                    return new ArrayList<>();
                }
                Long[] sopIds = Arrays.stream(mesProductPlan.getSopId().split(","))
                        .map(String::trim)
                        .map(Long::parseLong)
                        .toArray(Long[]::new);
                return mesBaseAttachInfoService.selectMesBaseAttachInfoByAttachIds(sopIds);
            } else {
                return new ArrayList<>();
            }
        } catch (Exception e) {
            return new ArrayList<>();
        }
    }

    /**
     * 工单撤回
     *
     * @param mesProductOrder 生产工单
     * @return
     */
    @Override
    public int productOrderRecall(MesProductOrder mesProductOrder) {
        // 检验生产派工未下达状态的可进行工单撤回
        MesProductPlan mesProductPlan = new MesProductPlan();
        mesProductPlan.setProductOrderId(mesProductOrder.getProductOrderId());
        List<MesProductPlan> mesProductPlans = mesProductPlanMapper.selectMesProductPlanList(mesProductPlan);
        for (MesProductPlan productPlan : mesProductPlans) {
            if (!productPlan.getPlanStatus().equals(MesConstants.MES_PRODUCT_PLAN_STATUS_TO_DISPATCH)
                    || !productPlan.getPlanStatus().equals(MesConstants.MES_PRODUCT_PLAN_STATUS_FINISH)) {
                throw new ServiceException("此工单已派工，无法撤回！");
            }
        }
        mesProductOrder.setOrderStatus(MesConstants.RECALL);
        // 调用仓库接口取消锁定库存
        mesProductOrder.setStockLockFlag("0");
        return mesProductOrderService.updateMesProductOrder(mesProductOrder);
    }

    /**
     * 查询生产派工列表Join product_order和base_material
     *
     * @param mesProductPlan 生产派工
     * @return 生产派工
     */
    @Override
    public List<MesProductPlan> selectMesProductPlanJoinList(MesProductPlan mesProductPlan) {
        Long stationId = SecurityUtils.getStationId();
        mesProductPlan.setStationId(stationId);
        return mesProductPlanMapper.selectMesProductPlanJoinList(mesProductPlan);
    }


    /**
     * 获取图纸列表
     *
     * @param mesProductPlan
     * @return
     */
    @Override
    public List<MesBaseAttachInfo> getDrawings(MesProductPlan mesProductPlan) {
        String attachId = mesProductPlan.getAttachId();
        List<MesBaseAttachInfo> baseAttachInfos;
        if (attachId != null) {
            MesBaseAttachInfo queryBaseAttachInfo = new MesBaseAttachInfo();
            queryBaseAttachInfo.setAttachIdStr(attachId);
            baseAttachInfos = mesBaseAttachInfoMapper.selectMesBaseAttachInfoList(queryBaseAttachInfo);
            MesProductPlanDetail queryProductPlanDetail = new MesProductPlanDetail();
            queryProductPlanDetail.setPlanId(mesProductPlan.getPlanId());
            List<MesProductPlanDetail> productPlanDetails = productPlanDetailMapper.selectMesProductPlanDetailList(queryProductPlanDetail);
            if (productPlanDetails != null && !productPlanDetails.isEmpty() && baseAttachInfos != null && !baseAttachInfos.isEmpty()) {
                productPlanDetails.forEach(productPlanDetail -> {
                    for (MesBaseAttachInfo baseAttachInfo : baseAttachInfos) {
                        if (productPlanDetail.getAttachId() != null && productPlanDetail.getAttachId().equals(baseAttachInfo.getAttachId())) {
                            baseAttachInfo.setDispatchFlag(true);
                        }
                    }
                });
            }

            return baseAttachInfos;
        }

        return null;
    }

}
